@@ -488,9 +488,11 @@ sub on_keydown
 		utf8::upgrade($chr) if $is_unicode;
 		my ($curpos, $advance);
 		if ( $p_start != $p_end) {
-			substr( $cap, $p_start, $p_end - $p_start) = $chr;
-			$self-> edit_text( $cap);
+			substr( $cap, $p_start, $p_end - $p_start) = '';
 			$self-> charOffset($self->{glyphs}->index2cluster($p_start));
+			$self-> edit_text( $cap);
+			local $self->{insertMode} = 1;
+			$self-> handle_input($chr);
 		} else {
 			$self-> handle_input($chr);
 		}
@@ -795,7 +797,7 @@ sub on_fontchanged
 	$self-> {font_width} = $font-> width;
 
 	$self-> check_auto_size;
-	$self-> reset;
+	$self-> text($self-> text);
 }
 
 sub set_alignment
@@ -962,7 +964,7 @@ sub set_selection
 			my ( $s, $e ) = $self-> {glyphs}-> selection2range($start, $end - 1);
 			$self->{selTextStart} = $s;
 			$self->{selTextEnd}   = $e + 1;
-			$self->{selChunks} = $self->{glyphs}->selection_chunks($s,$e);
+			$self->{selChunks} = $self->{glyphs}->selection_chunks_clusters($s,$e);
 		}
 		$new_chunks = $self->{selChunks};
 	} else {
