@@ -138,6 +138,7 @@ sub init
 	my %font = %{$self-> fontPalette-> [0]};
 	$font{pitch} = fp::Fixed;
 	$self-> {fontPalette}-> [1] = \%font;
+	$self-> {fontPaletteSize} = 2;
 
 	$self-> $_($profile{$_}) for qw( styles images pageName topicView);
 
@@ -1304,10 +1305,11 @@ sub format
 	# cache indents
 	my @indents;
 	my $state = $self-> create_state;
-	for ( 0 .. ( scalar @{$self-> fontPalette} - 1)) {
-		$$state[ tb::BLK_FONT_ID] = $_;
+
+	for my $fid ( 0 .. ( scalar @{$self-> fontPalette} - 1)) {
+		$$state[ tb::BLK_FONT_ID] = $fid;
 		$self-> realize_state( $self, $state, tb::REALIZE_FONTS);
-		$indents[$_] = $self-> font-> width;
+		$indents[$fid] = $self-> font-> width;
 	}
 	$$state[ tb::BLK_FONT_ID] = 0;
 
@@ -1341,8 +1343,10 @@ sub format
 
 	while ( 1) {
 		$self-> format_chunks;
-		last unless $self-> {formatData} && $self-> {blocks}-> [-1] &&
-						$self-> {blocks}-> [-1]-> [tb::BLK_Y] < $ph;
+		last unless 
+			$self-> {formatData} && 
+			$self-> {blocks}-> [-1] &&
+			$self-> {blocks}-> [-1]-> [tb::BLK_Y] < $ph;
 	}
 }
 
@@ -1379,6 +1383,8 @@ sub format_chunks
 		push @$g, @$m[ M_START .. $#$m ];
 
 		# format the paragraph
+
+		my $next_text_offs = ( $mid == $#{$self->{model}} ) ? length( ${$self->{text}} ) : $self->{model}->[$mid + 1]->[M_TEXT_OFFSET];
 		my $indent = $$m[M_INDENT] * $$indents[ $$m[M_FONT_ID]];
 		@blocks = $self-> block_wrap( $self, $g, $state, $formatWidth - $indent);
 
@@ -1510,9 +1516,12 @@ sub print
 	my $pageno = 1;
 	my $pagenum  = sub {
 		$canvas->translate( 0, 0 );
+		my %save = %{$canvas->font};
 		$canvas->font->set( name => $self->fontPalette->[0]->{name} || 'Default', size => 6, style => 0, pitch => fp::Default );
 		$canvas->set( color => cl::Black );
 		$canvas->text_out( $pageno, ( $formatWidth - $canvas->get_text_width($pageno) ) / 2, ($vmargin - $canvas->font->height ) / 2 );
+		delete $save{height}; # XXX fix this
+		$canvas->font(\%save);
 		$pageno++;
 	};
 	my $new_page = sub {
