@@ -52,6 +52,7 @@ typedef struct {
 	List   vectors;
 	Bool   ranges_queried;
 	Bool   is_active;
+	unsigned int flags;
 } PassiveFontEntry, *PPassiveFontEntry;
 
 #define PASSIVE_FONT(fid) ((PPassiveFontEntry) font_passive_entries.items[(unsigned int)(fid)])
@@ -151,7 +152,7 @@ query_ranges(PPassiveFontEntry pfe)
 	f.pitch = fpDefault;
 
 	pfe-> ranges_queried = true;
-	ranges = apc_gp_get_mapper_ranges(&f, &count);
+	ranges = apc_gp_get_mapper_ranges(&f, &count, &pfe->flags);
 	if ( count <= 0 ) {
 		list_create( &pfe->vectors, 0, 1);
 		return;
@@ -169,6 +170,12 @@ query_ranges(PPassiveFontEntry pfe)
 		for (j = from; j <= to; j++) {
 			Byte * map;
 			unsigned int page = j >> FONTMAPPER_VECTOR_BASE, bit = j & FONTMAPPER_VECTOR_MASK;
+
+			if ( !(pfe->flags & MAPPER_FLAGS_COMBINING_SUPPORTED)) {
+				if ( j >= 0x300 && j <= 0x36f )
+					continue;
+			} 
+
 			if (( map = (Byte*) pfe->vectors.items[page]) == NULL ) {
 				if (!( map = malloc(1 << (FONTMAPPER_VECTOR_BASE - 3)))) {
 					warn("Not enough memory");
@@ -229,7 +236,11 @@ can_substitute(uint32_t c, int pitch, int fid)
 	if ( !pfe-> ranges_queried )
 		query_ranges(pfe);
 
-	if ( pitch != fpDefault && (( pfe->font.undef.pitch || pfe->font.pitch != pitch )))
+	if ( 
+		pitch != fpDefault && 
+		(( pfe->font.undef.pitch || pfe->font.pitch != pitch )) &&
+		!( pfe-> flags & MAPPER_FLAGS_SYNTHETIC_PITCH)
+	)
 		return false;
 
 	page = c >> FONTMAPPER_VECTOR_BASE;
@@ -989,6 +1000,35 @@ analyze_fonts( Handle self, PTextShapeRec t, uint16_t * fonts)
 		unsigned int nfid = find_font(*(text++), pitch, fid);
 		if ( nfid != fid ) fonts[i] = nfid;
 	}
+
+	/* make sure that all combiners are same font as base glyph */
+	text = t->text;
+	for ( i = 0; i < t->len; ) {
+		int j, base, non_base_font = 0, need_align = 0;
+		if ( text[i] >= 0x300 && text[i] <= 0x36f ) {
+			i++;
+			continue;
+		}
+
+		base = i++;
+		for ( ; i < t->len; ) {
+			if ( text[i] < 0x300 || text[i] > 0x36f )
+				break;
+			if ( fonts[i] != fonts[base] ) {
+				need_align = 1;
+				non_base_font = fonts[i];
+				break;
+			}
+			i++;
+		}
+		if (!need_align) continue;
+
+		/* don't test all combiners for all fonts, such as f.ex. base glyph is Arial, ` is Courier, and ~ is Times.
+		   Wild guess is that if ` is supported, others should be too */
+		for ( j = base; j <= i; j++)
+			fonts[j] = non_base_font;
+
+	}
 }
 
 static Bool
@@ -1236,7 +1276,9 @@ Drawable_text_shape( Handle self, SV * text_sv, HV * profile)
 		int pitch = pget_i(pitch);
 		if ( pitch == fpVariable || pitch == fpFixed )
 			t.flags |= pitch << toPitch;
-	}
+	} else if ( var-> font. pitch == fpFixed )
+		t.flags |= fpFixed << toPitch;
+		
 
 	hv_clear(profile); /* old gencls bork */
 
