@@ -1365,7 +1365,7 @@ prima_xft_get_glyphs_width( PCachedFont self, PGlyphsOutRec t, Point * overhangs
 	int i, ret = 0;
 	FontContext fc;
 
-	font_context_init(&fc, &self->font, t->fonts, self->xft, NULL);
+	font_context_init(&fc, &self->font, t->fonts, self->xft_base, NULL);
 	if ( overhangs) overhangs-> x = overhangs-> y = 0;
 
 	t->len = check_width(self, t->len);
@@ -1745,11 +1745,12 @@ overstrike( Handle self, int x, int y, Point *ovx, int advance)
 
 	if ( ovx->x < 0 ) ovx->x = 0;
 	if ( ovx->y < 0 ) ovx->y = 0;
+	advance += ovx->y;
+
 	if ( PDrawable( self)-> font. style & fsUnderlined) {
 		ay = d;
 		x1 = x - ovx->x * c - ay * s + 0.5;
 		y1 = y - ovx->x * s + ay * c + 0.5;
-		advance += ovx->y;
 		x2 = x + advance * c - ay * s + 0.5;
 		y2 = y + advance * s + ay * c + 0.5;
 		XDrawLine( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y1), x2, REVERT( y2));
@@ -1759,7 +1760,6 @@ overstrike( Handle self, int x, int y, Point *ovx, int advance)
 		ay = (XX-> font-> font.ascent - XX-> font-> font.internalLeading)/2;
 		x1 = x - ovx->x * c - ay * s + 0.5;
 		y1 = y - ovx->x * s + ay * c + 0.5;
-		advance += ovx->y;
 		x2 = x + advance * c - ay * s + 0.5;
 		y2 = y + advance * s + ay * c + 0.5;
 		XDrawLine( DISP, XX-> gdrawable, XX-> gc, x1, REVERT( y1), x2, REVERT( y2));
@@ -2416,17 +2416,49 @@ prima_xft_get_glyph_outline( Handle self, int index, int flags, int ** buffer)
 }
 
 unsigned long *
-prima_xft_mapper_query_ranges(PFont font, int * count)
+prima_xft_mapper_query_ranges(PFont font, int * count, unsigned int * flags)
 {
 	char name[256];
 	XftFont * xft = NULL;
+	unsigned long * ranges;
 	strncpy(name, font->name, 256);
 	prima_xft_font_pick( nilHandle, font, font, NULL, &xft);
+	*flags = 0 | MAPPER_FLAGS_SYNTHETIC_PITCH;
 	if ( !xft || strcmp( font->name, name ) != 0 ) {
 		*count = 0;
 		return NULL;
 	}
-	return get_font_ranges( xft->charset, count);
+	ranges = get_font_ranges( xft->charset, count);
+#ifdef WITH_HARFBUZZ
+{
+	FT_Face face;
+	hb_buffer_t *buf;
+	hb_font_t *font;
+	hb_glyph_position_t *glyph_pos;
+	unsigned int l = 0;
+	uint32_t x_tilde[2] = {'x', 0x330};
+
+	if ( !( face = XftLockFace( xft)))
+		return ranges;
+
+	buf = hb_buffer_create();
+	hb_buffer_add_utf32(buf, x_tilde, 2, 0, -1);
+	hb_buffer_guess_segment_properties (buf);
+	font = hb_ft_font_create(face, NULL);
+	hb_shape(font, buf, NULL, 0);
+	glyph_pos  = hb_buffer_get_glyph_positions(buf, &l);
+
+	if ( l == 2 && glyph_pos[1].x_advance == 0 ) {
+		*flags |= MAPPER_FLAGS_COMBINING_SUPPORTED;
+	}
+
+	hb_buffer_destroy(buf);
+	hb_font_destroy(font);
+	XftUnlockFace(xft);
+
+}
+#endif
+	return ranges;
 }
 
 Bool
@@ -2435,7 +2467,7 @@ prima_xft_text_shaper( Handle self, PTextShapeRec r, uint32_t * map8)
         int i;
 	uint16_t *glyphs;
 	uint32_t *text;
-	XftFont *font = X(self)->font->xft;
+	XftFont *font = X(self)->font->xft_base;
 
         for ( 
 		i = 0, glyphs = r->glyphs, text = r->text;
